@@ -1,6 +1,6 @@
 # 2016. 1. 10 by Hans Roh hansroh@gmail.com
 
-__version__ = "0.7.6.27"
+__version__ = "0.7.7.3"
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
 import threading
 from . import lifetime, queue, request_builder, response_builder, stubproxy
@@ -85,32 +85,36 @@ def configure (
 	force_http1 = False,
 	http2_constreams = 1,
 	allow_redirects = True,
-	qrandom = False
+	qrandom = False,
+	use_pool = True,
+	backend = False,
 ):
-	global _logger, _cb_gateway, _concurrent, _initialized, _timeout, _workers, _que, _allow_redirects, _force_h1
+	global _logger, _cb_gateway, _concurrent, _initialized, _timeout
+	global _workers, _que, _allow_redirects, _force_h1
 	
 	if logger is None:
 		logger = logger_f.screen_logger ()
 	_logger = logger
-
+	
 	if qrandom:		
 		_que = queue.RandomQueue ()
 	else:
 		_que = queue.Queue ()
 	
 	_allow_redirects = allow_redirects
-	_force_h1 = request_handler.RequestHandler.FORCE_HTTP_11 = force_http1	
+	_force_h1 = request_handler.RequestHandler.FORCE_HTTP_11 = force_http1
+	
+	if not use_pool:
+		asynconnect.AsynConnect.keep_connect = use_pool
+		asynconnect.AsynSSLConnect.keep_connect = use_pool		
 	if not _force_h1:
 		asynconnect.AsynConnect.fifo_class = await_fifo
 		asynconnect.AsynSSLConnect.fifo_class = await_fifo
-		
+			
 	http2.MAX_HTTP2_CONCURRENT_STREAMS = http2_constreams
 	_workers = workers
 	_concurrent = workers
 	
-	#if _concurrent == 1:
-		# for preventing lifetime.loop break
-	#	trigger.start_trigger ()
 	if not force_http1:
 		_concurrent = workers * http2_constreams
 	elif http2_constreams:
@@ -125,19 +129,25 @@ def configure (
 	client.set_timeout (timeout)
 	dbapi.set_timeout (timeout)
 	
-	socketpool.create (_logger)
-	dbpool.create (_logger)
+	socketpool.create (_logger, backend = backend)
+	dbpool.create (_logger, backend = backend)
 	adns.init (_logger)
-	lifetime.init (_timeout / 2.) # maintern interval
+	lifetime.init (_timeout / 2., logger) # maintern interval
 	_initialized = True
 
+def _reque_first (request):
+	global _que	
+	_que.first (request)	
+
 def handle_status_401 (response):
 	global _que
 	if not response.request.get_auth () or response.request.reauth_count:
 		return response
-	response.request.reauth_count = 1		
-	_que.first (response.request, 0)	
+	response.request.reauth_count = 1
 	
+	_logger ("authorization failed, %s" % response.url, "info")
+	_reque_first (response.request)
+
 def handle_status_3xx (response):
 	global _allow_redirects	, _que
 	
@@ -145,48 +155,65 @@ def handle_status_3xx (response):
 		return response
 	if response.status_code not in (301, 302, 307, 308):
 		return response
-	
+
 	newloc = response.get_header ('location')	
-	oldloc = response.request.uri	
+	oldloc = response.request.uri
+	original_request = response.request
+	
+	if newloc == oldloc:
+		response.response = http_response.FailedResponse (711, "Redirect Error", original_request)
+		return response
 	
 	try:
-		response.request.relocate (response.response, newloc)
+		new_request = response.request.relocate (response.response, newloc)
 	except RuntimeError:		
-		response.response = http_response.FailedResponse (711, "Redirect Error", response.request)
+		response.response = http_response.FailedResponse (711, "Redirect Error", original_request)
 		return response
-		
-	_logger ("%s redirected %s from %s" % (response.status_code, response.reason, oldloc), "info")
-	_que.first (response.request, 0)
+	
+	_logger ("%s redirected to %s from %s" % (response.status_code, newloc, oldloc), "info")	
+	# DO NOT use relocated response.request, it is None
+	_reque_first (new_request)
 	
 def _request_finished (handler):
 	global _cb_gateway, _currents, _concurrent, _finished_total, _logger, _bytesrecv,_force_h1
-	
-	if isinstance (handler, dbo_request.Request):
-		response = handler
-		_currents.pop (response.meta ['req_id'])
-		
-	else:
-		response = response_builder.HTTPResponse (handler)
-		_currents.pop (response.meta ['req_id'])				
-		for handle_func in (handle_status_401, handle_status_3xx):
-			response = handle_func (response)
-			if not response:			
-				return		
-	_finished_total += 1
-	response.logger = _logger
-	_bytesrecv += len (response.content)
-	callback = response.meta ['req_callback'] or _cb_gateway
 	try:
-		callback (response)
-	except:		
-		_logger.trace ()	
+		if isinstance (handler, dbo_request.Request):
+			response = handler
+			_currents.pop (response.meta ['req_id'])
+			
+		else:
+			response = response_builder.HTTPResponse (handler)
+			_currents.pop (response.meta ['req_id'])
+			for handle_func in (handle_status_401, handle_status_3xx):
+				response = handle_func (response)
+				if not response:
+					return		
+		_finished_total += 1
+		response.logger = _logger
+		_bytesrecv += len (response.content)
+		callback = response.meta ['req_callback'] or _cb_gateway
+		try:
+			callback (response)
+		except:		
+			_logger.trace ()	
 	
-	qsize () and _req ()
+	except RecursionError:
+		try: 
+			_currents.pop (handler.request.meta ['req_id'])
+		except KeyError: 
+			pass	
+		_logger ("too many error occured, stop requeueing", "info")	
+		
+	else:	
+		qsize () and _req ()
 		
 def _req ():
-	global _que, _logger, _currents, _request_total
+	global _que, _logger, _currents, _request_total, _backend
 	args = _que.get ()
 	
+	if args is None:
+		return
+		
 	_request_total += 1
 	_is_request = False
 	_is_db = False
@@ -225,13 +252,16 @@ def _req ():
 			asyncon = socketpool.get (req.uri)		
 		_currents [meta ['req_id']] = [0, req.uri]
 		
-		asyncon.prevent_recursion = True
 		handler = req.handler (asyncon, req, _request_finished)
-		if asyncon.get_proto () and asyncon.isconnected ():			
+		if asyncon.get_proto () and asyncon.isconnected ():
 			asyncon.handler.handle_request (handler)
 		else:
 			handler.handle_request ()	
-		
+
+def workings ():
+	global _currents
+	return 	len (_currents)
+			
 def countreq ():
 	global _request_total
 	return 	_request_total
@@ -256,12 +286,14 @@ def concurrent ():
 	return _concurrent
 
 def fetchall ():
-	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns	
+	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns, _force_h1	
 	
 	if not _initialized:
 		configure ()
 	
 	_fetch_started = timeit.default_timer ()
+	# IMP. mannually set
+	lifetime._polling = 1
 	
 	# create initail workers	
 	_logger ("creating connection pool", "info")
@@ -269,9 +301,9 @@ def fetchall ():
 	for i in range (target_socks):
 		_req ()
 			
-	if not request_handler.RequestHandler.FORCE_HTTP_11 and http2.MAX_HTTP2_CONCURRENT_STREAMS > 1:		
+	if not _force_h1 and http2.MAX_HTTP2_CONCURRENT_STREAMS > 1:
 		# wait all availabale	
-		while 1:
+		while qsize ():
 			lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
 			if sum ([1 for conn in asyncore.socket_map.values () if not isinstance (conn, async_dns) and conn.get_proto () in H2_PROTOCOLS and conn.connected and not conn.isactive ()]) == _workers:
 				_logger ('%d connection(s) created' % target_socks, 'info')
@@ -282,10 +314,16 @@ def fetchall ():
 		while _concurrent > min (len (_currents), mapsize ()) and qsize ():
 			_req ()			
 		_max_conns = max (_max_conns, mapsize ())	
-		#print ('---', _concurrent, len (_currents), mapsize (), qsize ())
-		lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)			
-		
-	_duration = timeit.default_timer () - _fetch_started	
+		#print ('--', len (_currents), mapsize (), qsize ())
+		if not mapsize ():
+			break
+		lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
+	
+	#for each in _currents:
+	#	print ('-- unfinished', each)
+	
+	lifetime._polling = 0	
+	_duration = timeit.default_timer () - _fetch_started
 	socketpool.cleanup ()
 	dbpool.cleanup ()
 	
@@ -307,12 +345,8 @@ def suspend (timeout):
 
 _dns_reqs = 0
 def _add (method, url, params = None, auth = None, headers = {}, callback = None, meta = None, proxy = None):	
-	global _que, _initialized, _dns_query_req, _dns_reqs
-	
-	def dns_result (answer = None):
-		global _dns_reqs		
-		_dns_reqs -= 1
-	
+	global _que, _initialized, _dns_query_req, _dns_reqs, _workers
+
 	if not _initialized:		
 		configure ()
 		
@@ -325,12 +359,13 @@ def _add (method, url, params = None, auth = None, headers = {}, callback = None
 	host = urlparse (url) [1].split (":")[0]
 	
 	# DNS query for caching and massive
-	if not lifetime._polling:	
-		if host not in _dns_query_req:
+	if not lifetime._polling:
+		if _dns_reqs < _workers and host not in _dns_query_req:
 			_dns_query_req [host] = None
 			_dns_reqs += 1
-			adns.query (host, "A", callback = dns_result)			
-		lifetime.poll_fun_wrap (0.1)
+			adns.query (host, "A", callback = lambda x: None)		
+		if mapsize ():
+			lifetime.poll_fun_wrap (0.05)
 		
 	_que.add ((method, url, params, auth, headers, meta, proxy))
 	
