@@ -27,11 +27,11 @@ class AsynConnect (asynchat.async_chat):
 	ac_out_buffer_size = 65535
 	zombie_timeout = 10
 	keep_alive = 10
+	backend_keep_alive = 300
 	request_count = 0
 	active = 0
 	fifo_class = deque
-	prevent_recursion = False
-	unused_error_port = 44413
+	keep_connect = True
 	
 	def __init__ (self, address, lock = None, logger = None):
 		self.address = address
@@ -48,12 +48,16 @@ class AsynConnect (asynchat.async_chat):
 		
 		self.initialize_connection ()
 		self._closed = False
+		self.backend = False
 		
 		self.ac_in_buffer = b''
 		self.incoming = []
 		self.producer_fifo = self.fifo_class ()
 		asyncore.dispatcher.__init__(self)
 	
+	def __repr__ (self):
+		return "<AsynConnect %s:%d>" % self.address
+		
 	def duplicate (self):
 		new_asyncon = self.__class__ (self.address, self.lock, self.logger)
 		# used in socketpool
@@ -61,11 +65,14 @@ class AsynConnect (asynchat.async_chat):
 		# used in skitai cluster manager
 		new_asyncon.keep_alive = self.keep_alive
 		new_asyncon.auth = self.auth
+		new_asyncon.backend = self.backend
+		new_asyncon.keep_connect = self.keep_connect
 		return new_asyncon
-			
-	def __repr__ (self):
-		return "<AsynConnect %s:%d>" % self.address
 	
+	def set_backend (self, flag = True):
+		self.backend = flag
+		self.keep_alive = self.backend_keep_alive
+		
 	def set_auth (self, auth):
 		self.auth = auth
 	
@@ -88,15 +95,15 @@ class AsynConnect (asynchat.async_chat):
 		self._proto = None
 		self._closed = True
 			
-		if not self.handler:			
+		if not self.handler:
 			# return to the pool
 			return self.set_active (False)		
 		if not self.errcode:
 			# disconnect intentionally
 			return
-		
+			
 		handler, self.handller = self.handler, None
-		keep_active = False			
+		keep_active = False
 		try:
 			keep_active = handler.connection_closed (self.errcode, self.errmsg)
 		except: 
@@ -115,9 +122,12 @@ class AsynConnect (asynchat.async_chat):
 	def end_tran (self):
 		if DEBUG:
 			self.logger ('end_tran {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
-		self.del_channel ()
+		if not self.backend:
+			self.del_channel ()
 		self.handler = None
-		self.set_active (False)		
+		self.set_active (False)
+		if not self.keep_connect:
+			self.disconnect ()
 				
 	def use_sendlock (self):
 		self.__sendlock = threading.Lock ()
@@ -145,7 +155,6 @@ class AsynConnect (asynchat.async_chat):
 		self._proto = None
 		self._handshaking = False
 		self._handshaked = False
-		self._dns_error = False
 		
 		self.established = False		
 		self.upgraded = False		
@@ -249,17 +258,12 @@ class AsynConnect (asynchat.async_chat):
 			try: asynchat.async_chat.connect (self, self.address)
 			except:	self.handle_error (714)
 			return
-				
-		answer = answer or adns.get (self.address [0], "A")
+		
 		ipaddr = answer and answer [-1]["data"] or None		
 		port = self.address [1]
 		
-		if not ipaddr:
-			if not self.prevent_recursion:
-				return self.handle_close (704)					
-			ipaddr = "127.0.0.1"
-			port = self.unused_error_port
-			self._dns_error = True
+		if not ipaddr:			
+			return self.handle_close (704)			
 												
 		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
 		try: asynchat.async_chat.connect (self, (ipaddr, port))									
@@ -314,35 +318,20 @@ class AsynConnect (asynchat.async_chat):
 		self.zombie_timeout = timeout
 		
 	def handle_connect (self):
-		if self._dns_error:
-			# oops, used, really?
-			self.unused_error_port += 1
-			return self.handle_close (704, "DNS Not Found")
-			
 		if hasattr (self.handler, "has_been_connected"):		
 			self.handler.has_been_connected ()
-	
-	def handle_dns_error (self):
-		self._dns_error = False
-		self.handle_close (704, "DNS Not Found")
 					
 	def handle_expt (self):
-		if self._dns_error:
-			return self.handle_dns_error ()			
 		self.handle_close (703)
 	
 	def handle_error (self, code = 701):
-		if self._dns_error:
-			return self.handle_dns_error ()			
 		self.trace ()
 		self.handle_close(code)
 	
 	def handle_timeout (self):		
 		self.handle_close (702)
 		
-	def handle_expt_event(self):
-		if self._dns_error:
-			return self.handle_dns_error ()	
+	def handle_expt_event (self):		
 		err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 		if err != 0:
 			self.handle_close (703, "Socket %d Error" % err)
@@ -428,12 +417,12 @@ class AsynConnect (asynchat.async_chat):
 		
 		# IMP: call add_channel () AFTER push()	otherwise threading issue will be raised
 		try:
-			if self.connected:
-				#should keep order but it seems meaningless? LOOK LATER with initiate_send
-				#self.initiate_send ()
-				self.add_channel ()
-			else:
+			if not self.connected:				
 				self.connect ()
+								
+			elif not self.backend:
+				self.add_channel ()
+				
 		except:
 			self.handle_error ()
 
@@ -504,7 +493,7 @@ class AsynSSLConnect (AsynConnect):
 					raise BlockingIOError				
 				except NameError:
 					raise socket.error (EWOULDBLOCK)
-													
+
 			# closed connection
 			elif why.errno in (ssl.SSL_ERROR_ZERO_RETURN, ssl.SSL_ERROR_EOF):
 				self.handle_close (700, "Connection closed by SSL_ERROR_ZERO_RETURN or SSL_ERROR_EOF")
