@@ -1,17 +1,18 @@
 import struct
 from .producers import grpc_producer
-from aquests.protocols.http.request import XMLRPCRequest
+from ..http.request import XMLRPCRequest
 
 class GRPCRequest (XMLRPCRequest):
 	initial_http_version = "2.0"
 	
-	def __init__ (self, uri, method, params = (), headers = None, auth = None, logger = None, meta = {}):
+	def __init__ (self, uri, method, params = (), headers = None, auth = None, logger = None, meta = {}, http_version = "2.0"):
 		self.uri = uri
 		self.method = method
 		self.params = params
 		self.auth = (auth and type (auth) is not tuple and tuple (auth.split (":", 1)) or auth)
 		self.logger = logger
 		self.meta = meta
+		self.http_version = http_version
 		self.address, self.path = self.split (uri)
 	
 		self.headers = {
@@ -26,7 +27,7 @@ class GRPCRequest (XMLRPCRequest):
 			self.method = "GET"
 		else:
 			self.headers ["Content-Type"] = "application/grpc"		
-	
+		
 	def get_cache_key (self):
 		return None
 		
@@ -42,9 +43,6 @@ class GRPCRequest (XMLRPCRequest):
 		self.uri += self.method
 		return (host, port), path
 	
-	def serialize (self):
+	def serialize (self):		
 		return grpc_producer (self.params [0])
-	
-	def get_content_length (self):
-		return self.payload.get_content_length ()
-		
+	
\ No newline at end of file
